@@ -1,21 +1,60 @@
 import os
 import collections
 import threading
-from .grpc import audio_helpers
 import sounddevice as sd
+from .grpc import audio_helpers
+from .grpc.audio_helpers import (
+    DEFAULT_AUDIO_SAMPLE_RATE, # 16000
+    DEFAULT_AUDIO_SAMPLE_WIDTH, # 2
+    DEFAULT_AUDIO_DEVICE_BLOCK_SIZE # 6400
+)
 
-class VirtualDevice:
-    def __init__ (self, smaple_rate = 16000, sample_width = 2, block_size = 6400):
-        self._sample_rate = smaple_rate
+class InputStreamBase:
+    def __init__(self, sample_rate = DEFAULT_AUDIO_SAMPLE_RATE, sample_width = DEFAULT_AUDIO_SAMPLE_WIDTH, block_size = DEFAULT_AUDIO_DEVICE_BLOCK_SIZE):
+        if sample_width != 2:
+            raise Exception('unsupported sample width:', sample_width)
+        self._sample_rate = sample_rate
         self._sample_width = sample_width
         self._block_size = block_size
 
+    def read (self, size):
+        raise NotImplementedError
+
+    def start (self):
+        raise NotImplementedError
+
+    def stop (self):
+        raise NotImplementedError
+
+    def close (self):
+        raise NotImplementedError
+
+    def write (self):
+        raise AttributeError
+    flush = write
+
+
+class SoundDeviceInputStream (audio_helpers.SoundDeviceStream, InputStreamBase):
+    def __init__(self, sample_rate = DEFAULT_AUDIO_SAMPLE_RATE, sample_width = DEFAULT_AUDIO_SAMPLE_WIDTH, block_size = DEFAULT_AUDIO_DEVICE_BLOCK_SIZE, device = None):
+        InputStreamBase.__init__ (self, sample_rate, sample_width, block_size)
+        self._audio_stream = sd.RawInputStream (
+            samplerate = sample_rate, dtype = 'int16', channels = 1, device = device,
+            blocksize = int (block_size/2)  # blocksize is in number of frames.
+        )
+
+
+class VirtualDeviceInputStream (InputStreamBase):
+    def __init__ (self, sample_rate = DEFAULT_AUDIO_SAMPLE_RATE, sample_width = DEFAULT_AUDIO_DEVICE_BLOCK_SIZE, block_size = DEFAULT_AUDIO_DEVICE_BLOCK_SIZE):
+        super ().__init__ (sample_rate, sample_width, block_size)
         self._lock = threading.Condition ()
         self._stopped = True
+        self._stand_by = False
         self._streams = collections.deque ()
 
     def put (self, stream):
         with self._lock:
+            if self._stopped:
+                return
             self._streams.append (stream)
             self._lock.notify ()
 
@@ -40,11 +79,15 @@ class VirtualDevice:
             assert len (data) == size
         return data
 
-    def flush(self):
-        pass
-
-    def start(self):
+    def get_ready (self):
         with self._lock:
+            self._stand_by = True
+
+    def start (self):
+        with self._lock:
+            if not self._stand_by:
+                # ignore start
+                return
             self._streams.clear ()
             self._stopped = False
             self._lock.notify ()
@@ -57,19 +100,3 @@ class VirtualDevice:
     def close (self):
         self.stop ()
 
-
-class SoundDeviceStream(audio_helpers.SoundDeviceStream):
-    def __init__(self, sample_rate, sample_width, block_size, flush_size, device = None):
-        if sample_width == 2:
-            audio_format = 'int16'
-        else:
-            raise Exception('unsupported sample width:', sample_width)
-        self._audio_stream = sd.RawInputStream(
-            samplerate=sample_rate, dtype=audio_format, channels=1, device = device,
-            blocksize=int(block_size/2)  # blocksize is in number of frames.
-        )
-        self._sample_width = sample_width
-        self._block_size = block_size
-        self._flush_size = flush_size
-        self._sample_rate = sample_rate
-
